import requests
from dotenv import load_dotenv
from datetime import datetime

load_dotenv()


def get_task_details(task, headers, taiga_url, cycle_times, cycle_time_data):
    task_history_url = f"{taiga_url}/history/task/{task['id']}"
    finished_date = task["finished_date"]
    try:
        response = requests.get(task_history_url, headers=headers)
        response.raise_for_status()
        history_data = response.json()
        in_progress_date = extract_new_to_in_progress_date(history_data)

        finished_date = datetime.fromisoformat(finished_date[:-1])
        if in_progress_date:
            in_progress_date = datetime.fromisoformat(
                str(in_progress_date)[:-6]
            )

            cycle_times.append({
                "taskId": task["id"],
                "taskDesc": task["subject"],
                "taskRef": task["ref"],
                "startTime": task["created_date"],
                "inProgressDate": in_progress_date.date(),
                "endTime": task['finished_date'],
                "endDate": finished_date.date(),
                "timeTaken": (finished_date - in_progress_date).days
            })
            cycle_time_data["closed_tasks"] += 1
            cycle_time_data["cycle_time"] += (
                finished_date - in_progress_date
            ).days

    except requests.exceptions.RequestException as e:
        print(f"Error fetching task by taskId: {e}")


def dev_focus_tasks(
    task,
    headers,
    taiga_url,
    from_date,
    to_date,
    dev_focus_data
):
    task_history_url = f"{taiga_url}/history/task/{task['id']}"
    try:
        response = requests.get(task_history_url, headers=headers)
        response.raise_for_status()
        history_data = response.json()
        in_progress_date = extract_new_to_in_progress_date(history_data)

        if in_progress_date:
            in_progress_date = datetime.fromisoformat(
                str(in_progress_date)[:-6]
            )

            if (
                in_progress_date > datetime.fromisoformat(from_date) and
                in_progress_date < datetime.fromisoformat(to_date)
            ):
                dev_focus_data.append({
                    "taskId": task["id"],
                    "taskDesc": task["subject"],
                    "taskRef": task["ref"],
                    "created_date": task["created_date"],
                    "inProgressDate": in_progress_date.date(),
                    "username": task["username"],
                    "full_name": task["full_name"]
                })

    except requests.exceptions.RequestException as e:
        print(f"Error fetching task by taskId: {e}")


def sorted_tasks_data(
    task,
    headers,
    taiga_url,
    from_date,
    to_date,
    member_tasks
):
    task_history_url = f"{taiga_url}/history/task/{task['id']}"
    try:
        response = requests.get(task_history_url, headers=headers)
        response.raise_for_status()
        history_data = response.json()
        in_progress_date, closed_date = extract_state_change_dates(
            history_data
        )
        if (in_progress_date is not None or closed_date is not None):
            in_progress_date = datetime.fromisoformat(
                str(in_progress_date)[:-6]
            ) if in_progress_date is not None else None
            closed_date = datetime.fromisoformat(
                str(closed_date)[:-6]
            ) if closed_date is not None else None

            if (
                (
                    in_progress_date is not None
                    and datetime.fromisoformat(from_date) < in_progress_date
                    < datetime.fromisoformat(to_date)
                ) or
                (
                    closed_date is not None
                    and datetime.fromisoformat(from_date) < closed_date
                    < datetime.fromisoformat(to_date)
                )
            ):
                task_data = {
                    "taskId": task["id"],
                    "taskDesc": task["subject"],
                    "taskRef": task["ref"],
                    "created_date": task["created_date"],
                    "inProgressDate": in_progress_date
                    if in_progress_date is not None
                    else datetime.fromisoformat(
                        task["created_date"]
                    ).replace(tzinfo=None),
                    "closed_date": closed_date,
                    "finished_date": task["finished_date"],
                    "username": task["username"],
                    "full_name": task["full_name"]
                }
                if task["username"] in member_tasks:
                    member_tasks[task["username"]].append(task_data)
                else:
                    member_tasks[task["username"]] = [task_data]

    except requests.exceptions.RequestException as e:
        print(f"Error fetching task by taskId: {e}")


def extract_new_to_in_progress_date(history_data):
    for event in history_data:
        values_diff = event.get("values_diff", {})
        if (
            "status" in values_diff and
            values_diff["status"] == ["New", "In progress"]
        ):
            created_at = datetime.fromisoformat(event["created_at"])
            return created_at
    return None


def extract_state_change_dates(history_data):
    in_progress_date, closed_at = None, None
    for event in history_data:
        values_diff = event.get("values_diff", {})
        if "status" in values_diff:
            if values_diff["status"] == ["New", "In progress"]:
                in_progress_date = datetime.fromisoformat(event["created_at"])
            if (values_diff["status"] == ['In progress', 'DONE'] or
                    values_diff["status"] == ['Ready for test', 'DONE']):
                closed_at = datetime.fromisoformat(event["created_at"])
    return in_progress_date, closed_at



import os

import requests
from dotenv import load_dotenv

# Load environment variables from .env file
load_dotenv()


# Function to retrieve user stories for a specific project from the Taiga API
def get_user_story(project_id, auth_token):

    # Get Taiga API URL from environment variables
    taiga_url = os.getenv('TAIGA_URL')
    # Construct the URL for the user stories API endpoint
    # for the specified project
    user_story_api_url = f"{taiga_url}/userstories?project={project_id}"

    # Define headers including the authorization token and content type
    headers = {
        'Authorization': f'Bearer {auth_token}',
        'Content-Type': 'application/json',
        "x-disable-pagination": "True"
    }

    try:

        # Make a GET request to Taiga API to retrieve user stories
        response = requests.get(user_story_api_url, headers=headers)
        # Raise an exception for HTTP errors (4xx or 5xx)
        response.raise_for_status()

        # Extract and return the user stories information from the response
        project_info = response.json()
        return project_info

    except requests.exceptions.RequestException as e:

        # Handle errors during the API request and print an error message
        print(f"Error fetching stories by slug: {e}")
        return None


# Method to return the details for closed user stories
def get_closed_user_stories(project_id, auth_token):

    user_stories = get_user_story(project_id, auth_token)
    if user_stories:

        closed_user_stories = [
            {
                "id": user_story["id"],
                "subject": user_story["subject"],
                "created_date": user_story["created_date"],
                "finished_date": user_story["finish_date"],
                "ref": user_story["ref"],
                "sprintURL": user_story["project_extra_info"]["slug"],
                "story_points": user_story["total_points"]
            }
            for user_story in user_stories if user_story.get("is_closed")
        ]

        return closed_user_stories
    else:
        return []


def get_user_story_details_by_id(
    story,
    headers,
    taiga_url
):
    task_history_url = f"{taiga_url}/history/userstory/{story['id']}"
    return requests.get(task_history_url, headers=headers)



import os
import requests
from dotenv import load_dotenv

load_dotenv()


def get_milestone(milstone_id, auth_token):
    taiga_url = os.getenv('TAIGA_URL')

    milestone_api_url = f"{taiga_url}/milestones/{milstone_id}"

    headers = {
        'Authorization': f'Bearer {auth_token}',
        'Content-Type': 'application/json',
        "x-disable-pagination": "True"
    }

    try:
        response = requests.get(milestone_api_url, headers=headers)

        response.raise_for_status()

        return response.json()

    except requests.exceptions.RequestException as e:
        print("Error fetching Milestone", e)
        return None



from unittest.mock import patch
from requests.exceptions import HTTPError

from ..project.getProjectBySlug \
    import get_project_by_slug


@patch('requests.get')
def test_get_project_by_slug_success(mock_get):
    mock_response = mock_get.return_value
    mock_response.raise_for_status.return_value = None
    mock_response.json.return_value = {"name": "Sample Project",
                                       "slug": "sample-project"}

    project_info = get_project_by_slug("sample-project", "fake-token")

    assert project_info is not None
    assert project_info["name"] == "Sample Project"
    assert project_info["slug"] == "sample-project"


@patch('requests.get')
def test_get_project_by_slug_failure(mock_get):
    mock_response = mock_get.return_value
    mock_response.raise_for_status.side_effect = HTTPError()

    project_info = get_project_by_slug("nonexistent-project", "fake-token")

    assert project_info is None



import os
import requests
from dotenv import load_dotenv

# Load environment variables from a .env file
load_dotenv()


# Function to retrieve project information by slug from the Taiga API
def get_project_by_slug(project_slug, auth_token):
    # Get Taiga API URL from environment variables
    taiga_url = os.getenv('TAIGA_URL')

    # Construct the URL for the project information API endpoint by slug
    project_api_url = f"{taiga_url}/projects/by_slug?slug={project_slug}"

    # Define headers including the authorization token and content type
    headers = {
        'Authorization': f'Bearer {auth_token}',
        'Content-Type': 'application/json',
    }

    try:
        # Make a GET request to Taiga API to retrieve project
        # information by slug
        response = requests.get(project_api_url, headers=headers)
        response.raise_for_status()

        # Extract and return the project information from the response
        project_info = response.json()
        return project_info

    except requests.exceptions.RequestException as e:
        # Handle errors during the API request and print an error message
        print(f"Error fetching project by slug: {e}")
        return None



import pytest

from unittest.mock import MagicMock, patch

from ..tasks.get_tasks import (
    get_tasks,
    get_closed_tasks,
    get_tasks_by_story_id,
    get_task_for_member
)


@pytest.fixture
def task():
    return [
        {
            "due_date": None,
            "due_date_reason": "",
            "due_date_status": "not_set",
            "total_comments": 0,
            "tags": [],
            "attachments": [],
            "project": 1521716,
            "project_extra_info": {
                "name": "SER516-Team-Cleveland",
                "slug": "ser516asu-ser516-team-cleveland",
                "logo_small_url": None,
                "id": 1521716
            },
            "status": 7600964,
            "status_extra_info": {
                "name": "In progress",
                "color": "#E47C40",
                "is_closed": False
            },
            "assigned_to": 618022,
            "assigned_to_extra_info": {
                "username": "AdvaitChirmule",
                "full_name_display": "Advait Chirmule",
                "photo": None,
                "big_photo": None,
                "gravatar_id": "5dafd57b749c0d2d00975cce81334152",
                "is_active": True,
                "id": 618022
            },
            "owner": 599211,
            "owner_extra_info": {
                "username": "sverma89",
                "full_name_display": "Shikha Verma",
                "photo": None,
                "big_photo": None,
                "gravatar_id": "d163ef3bd427c2473f561446d8397bca",
                "is_active": True,
                "id": 599211
            },
            "is_watcher": False,
            "total_watchers": 0,
            "is_voter": False,
            "total_voters": 0,
            "id": 5331122,
            "user_story": 5468188,
            "ref": 37,
            "milestone": 376615,
            "milestone_slug": "sprint1-1018",
            "created_at": "2021-01-20T16:03:15.369Z",
            "created_date": "2021-01-20T16:03:15.369Z",
            "modified_date": "2024-02-02T01:22:52.672Z",
            "finished_date": "2024-02-05T00:00:00Z",
            "subject": "FE Login Page",
            "us_order": 1706757804885,
            "taskboard_order": 0,
            "is_iocaine": False,
            "external_reference": None,
            "version": 4,
            "watchers": [],
            "is_blocked": False,
            "blocked_note": "",
            "is_closed": True,
            "username": "hmshahid",
            "full_name": "Hasan Shahid",
            "user_story_extra_info": {
                "id": 5468188,
                "ref": 12,
                "subject": "Select a Taiga project to apply it to",
                "epics": [
                    {
                        "id": 218050,
                        "ref": 3,
                        "subject": "Lead Time",
                        "color": "#D3AC50",
                        "project": {
                            "id": 1521716,
                            "name": "SER516-Team-Cleveland",
                            "slug": "ser516asu-ser516-team-cleveland"
                        }
                    }
                ]
            },
            "values_diff": {
                "status": [
                    "New",
                    "In progress"
                ],
                "taskboard_order": [
                    1610518445123,
                    1
                ]
            }
        }
    ]


@patch('requests.get')
def test_get_tasks(mock_get, task):
    mock_response = MagicMock()
    mock_response.json.return_value = task

    mock_get.return_value = mock_response

    result = get_tasks("sample", "auth")

    assert len(result) == 1


@patch('requests.get')
def test_closed_tasks(mock_get, task):
    mock_response = MagicMock()
    mock_response.json.return_value = task

    mock_get.return_value = mock_response

    result = get_closed_tasks("sample", "auth")

    assert len(result) == 1


@patch('requests.get')
def test_task_by_story_id(mock_get, task):
    mock_response = MagicMock()
    mock_response.json.return_value = task

    mock_get.return_value = mock_response

    result = get_tasks_by_story_id("storyId", "auth")

    assert len(result) == 1


@patch('requests.get')
def test_task_for_member(mock_get, task):
    mock_response = MagicMock()
    mock_response.json.return_value = task

    mock_get.return_value = mock_response

    result = get_task_for_member("sample", "member_id", "auth")

    assert len(result) == 1
